import { Drag } from './drag';
import domUtils from '@/utils/domUtils'
import _ from 'lodash'

//解析参数
function parseOptions(binding, vnode) {
  const options = binding.value || {};
  if (_.isEmpty(options)) {
    options.drag = vnode.data.attrs.drag;
    options.modal = vnode.data.attrs.modal;
    options.escClose = vnode.data.attrs['esc-close'];
  }
  if (options.modal !== false)
    options.modal = true;
  return options;
}

function handleEscClose(options, vnode) {
  document.removeEventListener('keydown', vnode.componentInstance.EscClose);
  if (options.escClose) {
    document.addEventListener('keydown', vnode.componentInstance.EscClose);
  }
}
//iview Modal组件非模态
export default {
  name: 'modal-ext',
  directive: {
    inserted(el, binding, vnode) {
      let $el = el;
      let options = parseOptions(binding, vnode);
      let target = '.ivu-modal';
      if (!options.modal)
        target = '.ivu-modal-content';
      if (options.drag)
        $el._dragInstance = new Drag($el, { target, handler: '.ivu-modal-header' });
      if (!options.modal) {
        domUtils.addClass($el, 'no-modal');
        let mainEl = $el.querySelector('.ivu-modal');
        let size = domUtils.getSize(mainEl);
        let left = (window.innerWidth - size.width) / 2;
        let top = (window.innerHeight - size.height) / 2;
        mainEl.style.left = left + 'px';
        mainEl.style.top = top + 'px';
      }
      handleEscClose(options, vnode);
    },
    update(el, binding, vnode) {
      let options = parseOptions(binding, vnode);
      handleEscClose(options, vnode);
    },
    unbind() {
      if (!this) return;
      this.el._dragInstance && this.el._dragInstance.destory();
    }
  }
}
